/*global define,window,list*/

define([
    'testsuite/qunit',
    'testsuite/sinon',
    'jquery',
    'de_epages/mediagallery/filemanager/list-view',
    'de_epages/mediagallery/filemanager/folder-model',
    'testsuite/qunit-fixture!de_epages/mediagallery/filemanager/list-view-test',

    '$ready!'
], function (QUnit, sinon, $, ListView, FolderModel, fixture) {
    'use strict';

    var xhr = sinon.useFakeXMLHttpRequest(),
        requests = sinon.requests = [];

    QUnit.module('de_epages/mediagallery/filemanager/list-view', {
        setup: function () {
            fixture.init();

            var filemanager = $('.filemanager'),
                data = {
                    "files": [
                        {
                            "width": 1000,
                            "extension": ".jpg",
                            "name": "epages.jpg",
                            "path": "/epages.jpg",
                            "height": 237,
                            "directory": "/MediaGallery",
                            "modified": 1200994146,
                            "size": 67089,
                            "fullpath": "../de_epages/mediagallery/filemanager/epages.jpg",
                            "selectable": 0,
                            "mimetype": "image/jpg",
                            "metadata" : {
                                "type" : "MediaGallery"
                            }
                        }

                    ],
                    "children": [
                        {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "test",
                                "type": "MediaGallery",
                                "path": "/neu/test"
                            },
                            "data": "test",
                            "state": null
                        },
                        {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "test2",
                                "type": "MediaGallery",
                                "path": "/neu/test2"
                            },
                            "data": "test2",
                            "state": null
                        },
                        {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "test3",
                                "type": "MediaGallery",
                                "path": "/neu/test3"
                            },
                            "data": "test3",
                            "state": null
                        },
                        {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "test4",
                                "type": "MediaGallery",
                                "path": "/neu/test4"
                            },
                            "data": "test4",
                            "state": null
                        },
                        {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "test5",
                                "type": "MediaGallery",
                                "path": "/neu/test5"
                            },
                            "data": "test5",
                            "state": null
                        },
                        {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "test6",
                                "type": "MediaGallery",
                                "path": "/neu/test6"
                            },
                            "data": "test6",
                            "state": null
                        },
                        {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "test7",
                                "type": "MediaGallery",
                                "path": "/neu/test7"
                            },
                            "data": "test7",
                            "state": null
                        }
                    ],
                    "metadata": {
                        "displayname": "neu",
                        "type": "MediaGallery",
                        "path": "/neu"
                    },
                    "data": "neu",
                    "state": null
                },
                model = new FolderModel(data);

            window.list = new ListView({
                model: model,
                maxRender: 5,
                multiSelect: false
            });
            filemanager.append(list.render().el);

            QUnit.strictEqual(
                $('.ep-mediagallery-listView').length,
                1,
                'Create list view and append it to filemanager'
            );

            list.on('openFolder', function (data) {
                QUnit.strictEqual(
                    data.displayname,
                    'test',
                    'Trigger event open folder'
                );
            });

        },
        teardown: fixture.destroy
    });

    QUnit.test('view methods', 12 + 1, function () {
        $.each([
            'initialize',
            'render',
            'setSelectable',
            'addFileView',
            'addFolderView',
            'changeOrder',
            'triggerOpenFolder',
            'renameTreeNode',
            'getSelectedElements',
            'selectableselected',
            'createNewFolder',
            'deleteItems'
        ], function (i, method) {
            QUnit.strictEqual(
                $.isFunction(list[method]),
                true,
                method + ' is a method of list view'
            );
        });
    });

    QUnit.asyncTest('view functionality', 8 + 2, function () {
        $('.ep-mediagallery-folderContainer').trigger('click');

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer.ui-selected').length,
            1,
            'Select first folder in list'
        );

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer').length,
            5,
            'There are 5 items rendered at first'
        );

        window.setTimeout(function () {
            $('.ep-mediagallery-scrollContainer').scrollTop(100);
            $('.ep-mediagallery-scrollContainer').trigger('scroll');

            QUnit.strictEqual(
                $('.ep-mediagallery-folderContainer').length,
                7,
                'After scrolling there are 7 items rendered'
            );

            $('.ep-mediagallery-folderContainer:first').trigger('dblclick');

            QUnit.strictEqual(
                list.getSelectedElements().length,
                1,
                'Get selected elements: 1 item is selected'
            );

            list.deleteItems();

            QUnit.strictEqual(
                $('.ep-mediagallery-folderContainer').length,
                6,
                'Delete selected folder'
            );

            $('.ep-mediagallery-fileContainer').trigger('click');


            QUnit.strictEqual(
                $('.ep-mediagallery-fileContainer.ui-selected').length,
                1,
                'Select first file in list'
            );

            list.deleteItems();

            QUnit.strictEqual(
                $('.ep-mediagallery-fileContainer').length,
                0,
                'Delete selected file'
            );

            list.createNewFolder();

            QUnit.strictEqual(
                $('.ep-mediagallery-folderContainer').length,
                7,
                'Create new folder'
            );

            QUnit.start();
        }, 500);

    });

});