/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/presentation/ui/togglesection-test',
    'jquery',
    'de_epages/presentation/ui/togglesection',
    "util/storage",

    'jquery/metaparse'
], function (QUnit, fixture, $, ep, storage) {

    QUnit.module("ep.ui.togglesection", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("ep.ui.togglesection", function () {

        $('.ep-js').metaparse();

        var toggleBar = ep('.toggle-bar:first'),
            toggleArea = ep('.ep-toggle-section-area:first'),
            storageKey = toggleBar.data('storageKey'),
            delay = 600,
            localStorage;

        QUnit.equal(
            toggleBar.data('uiPresentationUiTogglesection') instanceof $.Widget,
            true,
            "presentationUiTogglesection widget was created and instance of jQuery.Widget"
        );

        $.each(['_create', '_toggleArea', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.presentationUiTogglesection.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        localStorage = storage.localStorage('toggleareas');
        QUnit.equal(
            localStorage instanceof Object,
            true,
            "localStorage exists"
        );


        window.setTimeout(function () {
            QUnit.equal(
                toggleArea.is(':visible'),
                false,
                "toggle area is not visible initially"
            );

            localStorage = storage.localStorage('toggleareas');
            QUnit.equal(
                localStorage[storageKey].open,
                false,
                "localStorage indicates that toggle area is closed"
            );

            toggleBar.trigger('click');

            window.setTimeout(function () {
                QUnit.equal(
                    toggleArea.is(':visible'),
                    true,
                    "toggle area is visible after the first click"
                );

                localStorage = storage.localStorage('toggleareas');
                QUnit.equal(
                    localStorage[storageKey].open,
                    true,
                    "localStorage indicates that toggle area is open"
                );

                toggleBar.trigger('click');

                window.setTimeout(function () {
                    QUnit.equal(
                        toggleArea.is(':visible'),
                        false,
                        "toggle area is not visible after the second"
                    );

                    localStorage = storage.localStorage('toggleareas');
                    QUnit.equal(
                        localStorage[storageKey].open,
                        false,
                        "localStorage indicates that toggle area is closed"
                    );

                    // destroy widget
                    toggleBar.presentationUiTogglesection('destroy');
                    QUnit.equal(
                        toggleBar.data('uiPresentationUiTogglesection') instanceof $.Widget,
                        false,
                        "presentationUiTogglesection widget was destroyed"
                    );

                    QUnit.start();
                }, delay+50);

            }, delay+50);

        }, delay+50);

    });

});