/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/product/ui/tax-exceptions-test',
    'testsuite/sinon',
    'jquery',
    'de_epages/product/ui/tax-exceptions',

    '$ready!'
], function (QUnit, fixture, sinon, $, de_epages) {
    /* Mock ajax */
    ep.ajax = function(o) {
        var result = $.Deferred(),
            dataMapping;

        if (o.data) {
            if (o.data.ChangeAction === 'JSONSaveTaxClassMappingTest1') {
                dataMapping = JSON.parse(o.data.TaxClassMapping);

                // Start: check preselected data
                QUnit.equal(
                    dataMapping.preselected.length,
                    1,
                    "Ajax: one preselected areas were sent."
                );
                QUnit.equal(
                    dataMapping.preselected[0].taxClass.id,
                    taxClassCountries.preselected[dataMapping.preselected[0].countryId].taxClasses[newPreselectedTaxClassIndex].id,
                    "Ajax: correct changed tax class for preselected area was sent."
                );

                // Start: check optional data
                QUnit.equal(
                    dataMapping.optional.length,
                    2,
                    "Ajax: both chosen optional areas were sent."
                );
                QUnit.equal(
                    dataMapping.optional[1].countryId,
                    taxClassCountries.optional.sorted[newPreselectedTaxClassIndex],
                    "Ajax: correct tax area for added optional area was sent."
                );
                QUnit.equal(
                    dataMapping.optional[1].taxClass.id,
                    taxClassCountries.optional[taxClassCountries.optional.sorted[newPreselectedTaxClassIndex]].taxClasses[newOptionalTaxClassIndex].id,
                    "Ajax: correct tax class for added optional area was sent."
                );

                result.resolveWith(result, [{
                    "success": true,
                    "data": o.data
                }]);
            } else if (o.data.ChangeAction === 'JSONSaveTaxClassMappingTest2') {
                QUnit.ok(
                    o.data.TaxClassMapping !== undefined,
                    "Ajax request done."
                )
                // resolve request// resolve request
                result.resolveWith(result, [{
                    "success": true,
                    "data": o.data
                }]);
            }
        }

        return result;
    };

    // global variables
    var taxClassCountries = {
            "preselected": {
                "sorted": [250],
                "250": {
                    "countryId": 250,
                    "native": "France",
                    "taxMatrixId": 5449,
                    "taxClasses": [{
                        "id": 5446,
                        "name": "keine",
                        "alias": "none",
                        "taxRate": "0"
                    }, {
                        "id": 5442,
                        "name": "normal",
                        "alias": "normal",
                        "taxRate": "3"
                    }, {
                        "id": 5443,
                        "name": "ermäßigt",
                        "alias": "reduced",
                        "taxRate": "5"
                    }, {
                        "id": 5444,
                        "name": "reduced2",
                        "alias": "reduced2",
                        "taxRate": "7"
                    }, {
                        "id": 5445,
                        "name": "superreduced",
                        "alias": "superreduced",
                        "taxRate": "10"
                    }]
                }
            },
            "optional": {
                "sorted": [56, 276, 40, 616],
                "40": {
                    "countryId": 40,
                    "native": "Österreich",
                    "taxMatrixId": 5449,
                    "taxClasses": [{
                        "id": 5359,
                        "name": "keine",
                        "alias": "none",
                        "taxRate": "0"
                    }, {
                        "id": 5355,
                        "name": "normal",
                        "alias": "normal",
                        "taxRate": "8"
                    }, {
                        "id": 5356,
                        "name": "ermäßigt",
                        "alias": "reduced",
                        "taxRate": "9"
                    }, {
                        "id": 5357,
                        "name": "ermäßigt 2",
                        "alias": "reduced2",
                        "taxRate": "10"
                    }, {
                        "id": 5358,
                        "name": "ermäßigt 3",
                        "alias": "reduced3",
                        "taxRate": "11"
                    }]
                },
                "56": {
                    "countryId": 56,
                    "native": "Belgique / België",
                    "taxMatrixId": 5449,
                    "taxClasses": [{
                        "id": 5366,
                        "name": "keine",
                        "alias": "none",
                        "taxRate": "0"
                    }, {
                        "id": 5363,
                        "name": "normal",
                        "alias": "normal",
                        "taxRate": "5"
                    }, {
                        "id": 5364,
                        "name": "ermäßigt",
                        "alias": "reduced",
                        "taxRate": "6"
                    }, {
                        "id": 5365,
                        "name": "Zwischensatz",
                        "alias": "superreduced",
                        "taxRate": "7"
                    }]
                },
                "276": {
                    "countryId": 276,
                    "native": "Deutschland",
                    "taxMatrixId": 5449,
                    "taxClasses": [{
                        "id": 5452,
                        "name": "keine",
                        "alias": "none",
                        "taxRate": "0"
                    }, {
                        "id": 5450,
                        "name": "normal",
                        "alias": "normal",
                        "taxRate": "2"
                    }, {
                        "id": 5451,
                        "name": "ermäßigt",
                        "alias": "reduced",
                        "taxRate": "3"
                    }]
                },
                "616": {
                    "countryId": 616,
                    "native": "Polska",
                    "taxMatrixId": 5449,
                    "taxClasses": [{
                        "id": 5543,
                        "name": "keine",
                        "alias": "none",
                        "taxRate": "0"
                    }, {
                        "id": 5540,
                        "name": "normal",
                        "alias": "normal",
                        "taxRate": "15"
                    }, {
                        "id": 5541,
                        "name": "ermäßigt",
                        "alias": "reduced",
                        "taxRate": "20"
                    }, {
                        "id": 5542,
                        "name": "ermäßigt 2",
                        "alias": "reduced2",
                        "taxRate": "25"
                    }]
                }
            }
        },
        taxClassMappings = {
            "preselected": [{
                "countryId": "250",
                "taxClass": {
                    "id": 5445,
                    "taxRate": "10",
                    "alias": "superreduced",
                    "name": "super reduziert"
                },
                "taxMatrixId": 5449
            }],
            "optional": [{
                "countryId": "40",
                "taxClass": {
                    "id": 5356,
                    "taxRate": "8",
                    "alias": "reduced",
                    "name": "reduziert"
                },
                "taxMatrixId": 5449
            }]
        },

        addNewItem = function(taxExceptionsDialog) {
            // trigger click on add button
            taxExceptionsDialog.find('.ep-tax-exception-add-button').trigger('click');
            // get item wrapper
            taxExceptionsOptionalNewWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-optional-new');
            // chose first entry containing an area in selectbox
            taxExceptionsOptionalNewWrapper.find('.ep-tax-exceptions-optional-new-taxarea')[0].selectedIndex = 1;
            // trigger change event
            taxExceptionsOptionalNewWrapper.find('.ep-tax-exceptions-optional-new-taxarea').trigger('change');
        },

        newPreselectedTaxClassIndex,
        newOptionalTaxAreaIndex,
        newOptionalTaxClassIndex;

    // set module wide fixture property
    QUnit.fixtureVar = fixture;


    QUnit.test('de_epages/product/ui/tax-exceptions - check existence of tax-exceptions methods', 14, function() {
        $.each([
            '_create',
            '_createDialog',
            '_send',
            '_handleTaxAreaSelection',
            '_handleTaxClassSelection',
            '_handleAddButton',
            '_handleDeleteButton',
            '_updateExceptionCounter',
            '_handleTaxClassSelect',
            '_render',
            '_saveInitState',
            '_initializeState',
            'open',
            'destroy'
        ], function() {
            QUnit.ok(
                $.isFunction($.ui.productUiTaxExceptions.prototype[this]),
                this + ' is a method of the tax-exceptions widget'
            );
        });
    });


    QUnit.asyncTest("de_epages/product/ui/tax-exceptions - Test basic functionality of widget", function() {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $taxExceptions = $('.ep-tax-exceptions-link');
        // instantiate tax-exceptions widget
        $taxExceptions.productUiTaxExceptions({
            objectId: 4711,
            changeAction: 'JSONSaveTaxClassMapping',
            taxClassCountries: taxClassCountries,
            taxClassMappings: taxClassMappings,
            selectors: {
                taxClassSelect: '#' + 'TaxClass'
            },
            autoOpen: false,
            debug: true
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$taxExceptions.productUiTaxExceptions('instance'),
                true,
                'productUiTaxExceptions widget instance exists.'
            );

            QUnit.ok(
                $._data($taxExceptions[0], 'events').click.length === 1,
                'Click event for link is registered.'
            );
            QUnit.ok(
                $._data($taxExceptions[0], 'events').click[0].namespace === 'taxExceptionLink',
                'Click event is registered below namespace "taxExceptionLink".'
            );

            $taxExceptions.productUiTaxExceptions('destroy');
            // wait until widget is destroyed
            window.setTimeout(function() {
                QUnit.equal(
                    $taxExceptions.data('uiProductUiTaxExceptions') === undefined,
                    true,
                    'Widget was destroyed.'
                );

                QUnit.ok(
                    $._data($taxExceptions[0], 'events') === undefined,
                    'Click event for link was removed.'
                );

                fixture.destroy();
                QUnit.start();
            }, 300);
        }, 300);
    });

    QUnit.asyncTest("de_epages/product/ui/tax-exceptions - UI tests", function() {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $taxExceptions = $('.ep-tax-exceptions-link');

        newPreselectedTaxClassIndex = 1;
        newOptionalTaxAreaIndex = 1;
        newOptionalTaxClassIndex = 0;

        // instantiate tax-exceptions widget
        $taxExceptions.productUiTaxExceptions({
            objectId: 4711,
            changeAction: 'JSONSaveTaxClassMappingTest1',
            taxClassCountries: taxClassCountries,
            taxClassMappings: taxClassMappings,
            selectors: {
                taxClassSelect: '#' + 'TaxClass'
            },
            autoOpen: false,
            debug: true
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$taxExceptions.productUiTaxExceptions('instance'),
                true,
                'productUiTaxExceptions widget instance exists.'
            );

            var taxExceptionsDialog,
                taxExceptionsPreselectedWrapper,
                taxtExceptionsOptionalWrapper,
                taxExceptionsOptionalWrapperRow,
                taxtExceptionsOptionalNewWrapper,
                selectedId,
                sendSpy;

            // open dialog
            $taxExceptions.trigger('click');
            // register spy
            sendSpy = sinon.spy($taxExceptions.data('uiProductUiTaxExceptions'), '_send');

            // wait until dialog is open
            window.setTimeout(function() {
                taxExceptionsDialog = $('.ep-tax-exceptions-dialog');
                taxExceptionsPreselectedWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-preselected');
                taxExceptionsOptionalWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-optional');

                // Start: preselected exceptions
                // check number of preselected items
                QUnit.ok(
                    taxExceptionsPreselectedWrapper.find('.ep-tax-exceptions-row').length === 1,
                    'One preselected item exists'
                );
                // check tax area and tax class
                QUnit.equal(
                    taxExceptionsPreselectedWrapper.find('.ep-tax-exceptions-row .ep-tax-exceptions-col-taxarea div').text(),
                    taxClassCountries.preselected[taxClassMappings.preselected[0].countryId].native,
                    "Correct preselected tax area"
                );
                QUnit.equal(
                    Number(taxExceptionsPreselectedWrapper.find('.ep-tax-exceptions-row .ep-tax-exceptions-preselected-taxclass').val()),
                    taxClassMappings.preselected[0].taxClass.id,
                    "Correct preselected tax class selected"
                );

                // Start: optional exceptions
                // check number of optional items
                QUnit.ok(
                    taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-row').length === 1,
                    'One optional item exists'
                );
                // check tax area and tax class
                QUnit.equal(
                    Number(taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-row .ep-tax-exceptions-optional-taxarea').val()),
                    taxClassMappings.optional[0].countryId,
                    "Correct optional tax area selected"
                );
                QUnit.equal(
                    Number(taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-row .ep-tax-exceptions-optional-taxclass').val()),
                    taxClassMappings.optional[0].taxClass.id,
                    "Correct optional tax class selected"
                );

                // trigger click on add button
                taxExceptionsDialog.find('.ep-tax-exception-add-button').trigger('click');

                window.setTimeout(function() {
                    taxExceptionsOptionalNewWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-optional-new');

                    // check number of optional items
                    QUnit.ok(
                        taxExceptionsOptionalNewWrapper.find('.ep-tax-exceptions-row').length === 1,
                        'One new optional item was added'
                    );

                    // check if option with id 56 (first options) is in select list
                    // (will be removed when chosen later as new area)
                    QUnit.ok(
                        Array.prototype.slice.call(
                            taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-optional-taxarea')[0].options
                        ).filter(function(item, index) {
                            return Number(item.value) === taxClassCountries.optional.sorted[newPreselectedTaxClassIndex];
                        }).length === 1,
                        'Tax area with id ' + taxClassCountries.optional.sorted[newPreselectedTaxClassIndex] +' is an option in the select box.'
                    );
                    // check if tax class select is disabled
                    QUnit.equal(
                        taxExceptionsOptionalNewWrapper.find('.ep-tax-exceptions-col-taxclass select').prop('disabled'),
                        true,
                        'Tax class select box is disabled first.'
                    );

                    // chose first entry containing an area in selectbox
                    taxExceptionsOptionalNewWrapper.find('.ep-tax-exceptions-optional-new-taxarea')[0].selectedIndex = newOptionalTaxAreaIndex + 1;
                    // trigger change event
                    taxExceptionsOptionalNewWrapper.find('.ep-tax-exceptions-optional-new-taxarea').trigger('change');

                    // get wrapper again after re-render
                    taxExceptionsPreselectedWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-preselected');
                    taxExceptionsOptionalWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-optional');

                    // check number of optional items
                    QUnit.ok(
                        taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-row').length === 2,
                        'Two optional item exists'
                    );

                    // get last row
                    taxExceptionsOptionalWrapperRow = taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-row:last');
                    // get id of selected area
                    selectedId = Number(taxExceptionsOptionalWrapperRow.find('.ep-tax-exceptions-optional-taxarea').val());

                    // test chosen tax area (id and name)
                    QUnit.equal(
                        selectedId,
                        taxClassCountries.optional.sorted[newPreselectedTaxClassIndex],
                        'ID of chosen tax area is ' + taxClassCountries.optional.sorted[newPreselectedTaxClassIndex] +'.'
                    );
                    QUnit.equal(
                        taxExceptionsOptionalWrapperRow.find('.ep-tax-exceptions-optional-taxarea option')[0].text,
                        taxClassCountries.optional[taxClassCountries.optional.sorted[newPreselectedTaxClassIndex]].native,
                        'Name of chosen tax area is ' + taxClassCountries.optional[taxClassCountries.optional.sorted[newPreselectedTaxClassIndex]].native +'.'
                    );

                    // check if option with id 56 was removed as option from the select list
                    // after chosen it as new tax area
                    QUnit.ok(
                        Array.prototype.slice.call(
                            taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-optional-taxarea')[0].options
                        ).filter(function(item, index) {
                            return Number(item.value) === taxClassCountries.optional.sorted[newPreselectedTaxClassIndex];
                        }).length === 0,
                        'Tax area with id ' + taxClassCountries.optional.sorted[newPreselectedTaxClassIndex] +' was removed as option from the select box after chosen it as new tax area.'
                    );

                    // check if all tax classes are available for the chosen area (id 56)
                    QUnit.equal(
                        Array.prototype.slice.call(
                            taxExceptionsOptionalWrapperRow.find('.ep-tax-exceptions-optional-taxclass')[0].options
                        ).filter(function(item, index) {
                            // check if tax class id from select box equals to tax class from lookup object
                            return Number(item.value) === taxClassCountries.optional[selectedId].taxClasses[index].id;
                        }).length,
                        taxClassCountries.optional[selectedId].taxClasses.length,
                        'All tax classes for the selected area are available.'
                    );

                    // change tax class of preselected item
                    taxExceptionsPreselectedWrapper.find('.ep-tax-exceptions-row .ep-tax-exceptions-preselected-taxclass')[0].selectedIndex = newPreselectedTaxClassIndex;
                    taxExceptionsPreselectedWrapper.find('.ep-tax-exceptions-row .ep-tax-exceptions-preselected-taxclass').trigger('change');

                    // save changes
                    taxExceptionsDialog.find('.ep-button-save').trigger('click');
                    // wait
                    window.setTimeout(function() {
                        // check if spy in ease method was called
                        QUnit.ok(
                            sendSpy.callCount === 1,
                            '"_send" method was called ' + sendSpy.callCount + ' times.'
                        );

                        // destroy widget
                        $taxExceptions.productUiTaxExceptions('destroy');
                        // wait until widget is destroyed
                        window.setTimeout(function() {
                            QUnit.equal(
                                $taxExceptions.data('uiProductUiTaxExceptions') === undefined,
                                true,
                                'Widget was destroyed.'
                            );

                            fixture.destroy();
                            QUnit.start();
                        }, 300);
                    }, 300);
                }, 300);
            }, 300); // wait until animation ended
        }, 300);
    });


    QUnit.asyncTest("de_epages/product/ui/tax-exceptions - Test dialog content", function() {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $taxExceptions = $('.ep-tax-exceptions-link'),
            times = 3,
            snapShot,
            taxExceptionsDialog,
            taxExceptionsOptionalWrapper;

        // instantiate tax-exceptions widget
        $taxExceptions.productUiTaxExceptions({
            objectId: 4711,
            changeAction: 'JSONSaveTaxClassMappingTest2',
            taxClassCountries: taxClassCountries,
            taxClassMappings: taxClassMappings,
            selectors: {
                taxClassSelect: '#' + 'TaxClass'
            },
            autoOpen: true
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            // dialog opened automatically on start up
            QUnit.equal(
                $taxExceptions.data('uiProductUiTaxExceptions').dialog.is(':visible'),
                true,
                "Dialog opened automatically on start up."
            );

            taxExceptionsDialog = $('.ep-tax-exceptions-dialog');
            // take snap shot from dialog content
            snapShot = taxExceptionsDialog.text();

            // add new exception items
            for (var i=0; i<times; i++) {
                addNewItem(taxExceptionsDialog);
            }

            taxExceptionsOptionalWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-optional');

            // check number of optional items
            QUnit.ok(
                taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-row').length === times + 1,
                times + ' new optional items were added'
            );

            // delete first item
            taxExceptionsOptionalWrapper.eq(0).find('.ep-tax-exceptions-delete').trigger('click');
            window.setTimeout(function() {
                taxExceptionsOptionalWrapper = taxExceptionsDialog.find('.ep-tax-exceptions-optional');

                // check number of optional items
                QUnit.ok(
                    taxExceptionsOptionalWrapper.find('.ep-tax-exceptions-row').length === (times + 1) - 1,
                    (times - 1) + ' optional items remain'
                );

                // click cancel button
                taxExceptionsDialog.find('.ep-button-cancel').trigger('click');
                window.setTimeout(function() {
                    // open dialog again
                    $taxExceptions.trigger('click');
                    window.setTimeout(function() {

                        // debugger;
                        QUnit.equal(
                            taxExceptionsDialog.text(),
                            snapShot,
                            "Dialog content was resetted after *Cancel* was pressed."
                        );

                        // open dialog
                        $taxExceptions.trigger('click');
                        window.setTimeout(function() {
                            // add new exception items
                            for (var i=0; i<times; i++) {
                                addNewItem(taxExceptionsDialog);
                            }

                            // take snap shot from dialog content
                            snapShot = taxExceptionsDialog.text();

                            // save changes => dialog closes
                            taxExceptionsDialog.find('.ep-button-save').trigger('click');

                            // wait for saving
                            window.setTimeout(function() {
                                // dialog opened automatically on start up
                                QUnit.equal(
                                    $taxExceptions.data('uiProductUiTaxExceptions').dialog.is(':visible'),
                                    false,
                                    "Dialog was closed after saving."
                                );

                                // open dialog
                                $taxExceptions.trigger('click');

                                window.setTimeout(function() {
                                    // test snap shot => content of reopened dialog should be equal
                                    QUnit.equal(
                                        taxExceptionsDialog.text(),
                                        snapShot,
                                        "Dialog content is equal after saving the data."
                                    );

                                    $taxExceptions.productUiTaxExceptions('destroy');
                                    // wait until widget is destroyed
                                    window.setTimeout(function() {

                                        fixture.destroy();
                                        QUnit.start();
                                    }, 300);
                                }, 300);
                            }, 300);
                        }, 300);
                    }, 200);
                }, 200);
            }, 200);
        }, 300);
    });

    QUnit.asyncTest("de_epages/product/ui/tax-exceptions - Test external select box", function() {
        /* !!! FIXTURE - comment out next lines if fixture is used !!! */
        // clear HTML fragments and initiate fixture (if fixture is used)
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $taxExceptions = $('.ep-tax-exceptions-link'),
            $taxClassSelect = $('#' + 'TaxClass'),
            confirmationDialog,
            linkState;

        // instantiate tax-exceptions widget
        $taxExceptions.productUiTaxExceptions({
            objectId: 4711,
            changeAction: 'JSONSaveTaxClassMapping',
            taxClassCountries: taxClassCountries,
            taxClassMappings: taxClassMappings,
            selectors: {
                taxClassSelect: '#' + 'TaxClass'
            }
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            // select other tax class
            $taxClassSelect[0].selectedIndex = 0;
            $taxClassSelect.trigger('change');

            // get confirmation dialog
            confirmationDialog = $('.ep-tax-exceptions-alert');

            QUnit.equal(
                confirmationDialog.length,
                1,
                "Confirmation dialog was created"
            );
            QUnit.equal(
                confirmationDialog.is(':visible'),
                true,
                "Confirmation dialog is open"
            );

            // click ok => link gets disabled
            $('.ep-tax-exceptions-alert').find('.ep-button-apply').trigger('click');
            window.setTimeout(function() {
                // link is disabled
                QUnit.equal(
                    $taxExceptions.hasClass('ep-tax-exceptions-link-disabled'),
                    true,
                    "Link to open tax exceptions dialog is disabled."
                );

                // select old tax class again
                $taxClassSelect[0].selectedIndex = 2;
                $taxClassSelect.trigger('change');

                // link is enabled again
                linkState = $taxExceptions.hasClass('ep-tax-exceptions-link-disabled');
                QUnit.equal(
                    linkState,
                    false,
                    "Link to open tax exceptions dialog is disabled."
                );

                // select other tax class
                $taxClassSelect[0].selectedIndex = 0;
                $taxClassSelect.trigger('change');

                // click cancel => link stays as it was
                $('.ep-tax-exceptions-alert').find('.ep-button-cancel').trigger('click');

                window.setTimeout(function() {
                    QUnit.equal(
                        $taxExceptions.hasClass('ep-tax-exceptions-link-disabled'),
                        linkState,
                        "Link to open tax exceptions dialog is as it was before (no changes after Cancel)."
                    );

                    $taxExceptions.productUiTaxExceptions('destroy');
                    // wait until widget is destroyed
                    window.setTimeout(function() {
                        QUnit.equal(
                            $taxExceptions.data('uiProductUiTaxExceptions') === undefined,
                            true,
                            'Widget was destroyed.'
                        );

                        QUnit.ok(
                            $._data($taxExceptions[0], 'events') === undefined,
                            'Click event for link was removed.'
                        );

                        fixture.destroy();
                        QUnit.start();
                    }, 300);
                }, 200);
            }, 200);
        }, 300);
    });
});