/*global define,window*/
define([
    'testsuite/qunit',
    'jquery',
    'ep',
    'testsuite/qunit-fixture!de_epages/paybox/ui/payboxcancel-test',
    'de_epages/paybox/ui/payboxcancel',
    'testsuite/sinon',
], function(QUnit, $, ep, fixture, payboxcancel, sinon) {

    'use strict';
    QUnit.module('de_epages/paybox/ui/payboxcancel', {
        setup: fixture.init,
        teardown: fixture.destroy
    });
//
    QUnit.test('Check if the behaviour of cancel link is as expected for transaction status = STATUS_ACCEPTED ', 1, function() {

        var UIControlParameters = {
                Objectid: "12345",
                PaymentTypeAlias: "Paybox",
                TransStatus: "STATUS_ACCEPTED",
                DisplayCancel: $('#DisplayCancel'),
                CancelLink: $('#CancelLink')
        };

        payboxcancel(UIControlParameters );

        QUnit.strictEqual(
            $("#CancelLink").attr('class'),
            'Disabled',
            'Check that the link has the expected classes. We are waiting for the value Disabled in link class attribute'
        );
    });
//
    QUnit.test('Check if the behaviour of cancel link is as expected for transaction status = STATUS_FAILED ', 1, function() {

        var UIControlParameters = {
                Objectid: "12345",
                PaymentTypeAlias: "Paybox",
                TransStatus: "STATUS_FAILED",
                DisplayCancel: $('#DisplayCancel'),
                CancelLink: $('#CancelLink')
        };

        payboxcancel(UIControlParameters );

        QUnit.strictEqual(
            $("#CancelLink").attr('class'),
            'Disabled',
            'Check that the link has the expected classes. We are waiting for the value Disabled in link class attribute'
        );
    });
//
    QUnit.test('Check if the behaviour of cancel link is as expected for empty transaction status', 1, function() {

        var UIControlParameters = {
                Objectid: "12345",
                PaymentTypeAlias: "Paybox",
                TransStatus: "",
                DisplayCancel: $('#DisplayCancel'),
                CancelLink: $('#CancelLink')
        };

        payboxcancel(UIControlParameters );


        QUnit.strictEqual(
            $("#CancelLink").attr('class'),
            'Disabled',
            'Check that the link has the expected classes to do not be able to use it. We are waiting for the value Disable in link class attribute'
        );
    });
//
    QUnit.test('Check if the behaviour of cancel link is as expected for non-paybox payment methods ', 1, function() {

        var UIControlParameters = {
                Objectid: "12345",
                PaymentTypeAlias: "Visa",
                TransStatus: "",
                DisplayCancel: $('#DisplayCancel'),
                CancelLink: $('#CancelLink')
        };

        payboxcancel(UIControlParameters );

        QUnit.strictEqual(
            $("#DisplayCancel").is(":visible"),
            false,
            'Check that the div that contains the capture and cancel elements are not visible. We are waiting for the attribute visible of the above div at false'
        );
    });
//
    QUnit.test('Check if the behaviour of cancel link is as expected if we get an unexpected transaction status', 1, function() {

        ep.ajax = function(o) {
            var result = $.Deferred();

            result.resolve({
                "status": "ok",
                "message": "",
                "transstatus": "STATUS_FAILED",
            });

            return result;
        };

        var UIControlParameters = {
                Objectid: "12345",
                PaymentTypeAlias: "Paybox",
                TransStatus: "STATUS_PAID",
                DisplayCancel: $('#DisplayCancel'),
                CancelLink: $('#CancelLink'),
                CurrentWindow: this.mockWindow
        };

        payboxcancel(UIControlParameters );


        $('#CancelLink').trigger('click');
        $('#YesId').trigger('click');

        QUnit.strictEqual(
            $("#Response").html(),
            'Transaction Failed',
            'Check that the value of the dialog is the expected for a non expected transstatus. We are waiting for the word "Transaction Failed" at the dialog'
        );
    });
//
    QUnit.test('Check if the behaviour of cancel link is as expected if it goes wrong', 1, function() {

        ep.ajax = function(o) {
            var result = $.Deferred();

            result.resolve({
                "status": "error",
                "message": "salsitxa"
            });

            return result;
        };


        var UIControlParameters = {
                Objectid: "12345",
                PaymentTypeAlias: "Paybox",
                TransStatus: "STATUS_PAID",
                DisplayCancel: $('#DisplayCancel'),
                CancelLink: $('#CancelLink')
        };

        payboxcancel(UIControlParameters );

        $('.ep-cancelamount').trigger('click');
        $('#YesId').trigger('click');

        QUnit.strictEqual(
            $("#Response").html(),
            'Transaction Failed',
            'Check that the value of the dialog is the expected for a wrong execution. We are waiting for an error message in the dialog'
        );
    });
});
